"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var api_config_1 = tslib_1.__importDefault(require("../config/api-config"));
var Network = /** @class */ (function () {
    function Network() {
    }
    Network.get = function (url, queryParams, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var urlString;
            return tslib_1.__generator(this, function (_a) {
                urlString = Network.stringifyUrl(url, queryParams);
                options = tslib_1.__assign({ method: "GET", mode: "cors", credentials: "include" }, options);
                return [2 /*return*/, Network.crud(urlString, options)];
            });
        });
    };
    Network.getStatic = function (url, queryParams, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var urlString, environment, host;
            return tslib_1.__generator(this, function (_a) {
                urlString = Network.stringifyUrl(url, queryParams);
                environment = process.env.NODE_ENV || "development";
                host = urlString.startsWith("http")
                    ? ""
                    : api_config_1.default.STATIC_HOST[environment];
                urlString = host ? host + urlString : urlString;
                options = tslib_1.__assign({ method: "GET", mode: "cors", credentials: "include" }, options);
                return [2 /*return*/, Network.crud(urlString, options)];
            });
        });
    };
    Network.post = function (url, data, queryParams) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var urlString, options;
            return tslib_1.__generator(this, function (_a) {
                urlString = Network.stringifyUrl(url, queryParams);
                options = {
                    method: "POST",
                    body: JSON.stringify(data),
                    mode: "cors",
                    credentials: "include",
                    headers: {
                        "Content-Type": "application/json",
                    },
                };
                return [2 /*return*/, Network.crud(urlString, options)];
            });
        });
    };
    Network.put = function (url, data, queryParams) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var urlString, options;
            return tslib_1.__generator(this, function (_a) {
                urlString = Network.stringifyUrl(url, queryParams);
                options = {
                    method: "PUT",
                    body: JSON.stringify(data),
                    mode: "cors",
                    credentials: "include",
                    headers: {
                        "Content-Type": "application/json",
                    },
                };
                return [2 /*return*/, Network.crud(urlString, options)];
            });
        });
    };
    Network.delete = function (url, queryParams) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var urlString, options;
            return tslib_1.__generator(this, function (_a) {
                urlString = Network.stringifyUrl(url, queryParams);
                options = {
                    method: "DELETE",
                    noInject: true,
                };
                return [2 /*return*/, Network.crud(urlString, options)];
            });
        });
    };
    Network.crud = function (urlString, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var environment, host, response, ok, status, headers, body_1, body_2, body, err;
            var _a, _b, _c, _d, _e;
            return tslib_1.__generator(this, function (_f) {
                switch (_f.label) {
                    case 0:
                        environment = process.env.NODE_ENV || "development";
                        host = urlString.startsWith("http")
                            ? ""
                            : api_config_1.default.HOST[environment];
                        urlString = host ? host + urlString : urlString;
                        if (!options.headers) {
                            options.headers = {};
                        }
                        options.toggleLoader && options.toggleLoader((_a = {}, _a[urlString] = true, _a));
                        options.headers["Origin"] =
                            window.location.protocol + "//" + window.location.host;
                        return [4 /*yield*/, fetch(urlString, options)];
                    case 1:
                        response = _f.sent();
                        ok = response.ok, status = response.status, headers = response.headers;
                        if (!ok) return [3 /*break*/, 7];
                        if (!(headers.get("content-type").indexOf("application/json") !== -1)) return [3 /*break*/, 3];
                        return [4 /*yield*/, response.json()];
                    case 2:
                        body_1 = _f.sent();
                        options.toggleLoader && options.toggleLoader((_b = {}, _b[urlString] = false, _b));
                        return [2 /*return*/, { status: status, body: body_1 }];
                    case 3:
                        if (!(headers.get("content-type").indexOf("application/zip") !== -1 ||
                            headers.get("content-type").indexOf("application/pdf") !== -1)) return [3 /*break*/, 5];
                        return [4 /*yield*/, response.arrayBuffer()];
                    case 4:
                        body_2 = _f.sent();
                        options.toggleLoader && options.toggleLoader((_c = {}, _c[urlString] = false, _c));
                        return [2 /*return*/, { status: status, body: body_2 }];
                    case 5: return [4 /*yield*/, response.text()];
                    case 6:
                        body = _f.sent();
                        options.toggleLoader && options.toggleLoader((_d = {}, _d[urlString] = false, _d));
                        return [2 /*return*/, { status: status, body: body }];
                    case 7: return [4 /*yield*/, response.json()];
                    case 8:
                        err = _f.sent();
                        options.toggleLoader && options.toggleLoader((_e = {}, _e[urlString] = false, _e));
                        console.log(err);
                        throw new Error(err.message, err.error);
                }
            });
        });
    };
    Network.stringifyUrl = function (url, queryParams) {
        return url
            ? queryParams
                ? Object.keys(queryParams).reduce(function (acc, key) { return acc.concat("".concat(key, "=").concat(queryParams[key])); }, url + "?")
                : url
            : "";
    };
    return Network;
}());
exports.default = Network;
//# sourceMappingURL=network.js.map