"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.incrementIfOdd = exports.selectCount = exports.incrementByAmount = exports.decrement = exports.increment = exports.counterSlice = exports.incrementAsync = void 0;
var tslib_1 = require("tslib");
var toolkit_1 = require("@reduxjs/toolkit");
var counterAPI_1 = require("./counterAPI");
var initialState = {
    value: 0,
    status: "idle",
};
// The function below is called a thunk and allows us to perform async logic. It
// can be dispatched like a regular action: `dispatch(incrementAsync(10))`. This
// will call the thunk with the `dispatch` function as the first argument. Async
// code can then be executed and other actions can be dispatched. Thunks are
// typically used to make async requests.
exports.incrementAsync = (0, toolkit_1.createAsyncThunk)("counter/fetchCount", function (amount) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var response;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, (0, counterAPI_1.fetchCount)(amount)
                // The value we return becomes the `fulfilled` action payload
            ];
            case 1:
                response = _a.sent();
                // The value we return becomes the `fulfilled` action payload
                return [2 /*return*/, response.data];
        }
    });
}); });
exports.counterSlice = (0, toolkit_1.createSlice)({
    name: "counter",
    initialState: initialState,
    // The `reducers` field lets us define reducers and generate associated actions
    reducers: {
        increment: function (state) {
            // Redux Toolkit allows us to write "mutating" logic in reducers. It
            // doesn"t actually mutate the state because it uses the Immer library,
            // which detects changes to a "draft state" and produces a brand new
            // immutable state based off those changes
            state.value += 1;
        },
        decrement: function (state) {
            state.value -= 1;
        },
        // Use the PayloadAction type to declare the contents of `action.payload`
        incrementByAmount: function (state, action) {
            state.value += action.payload;
        },
    },
    // The `extraReducers` field lets the slice handle actions defined elsewhere,
    // including actions generated by createAsyncThunk or in other slices.
    extraReducers: function (builder) {
        builder
            .addCase(exports.incrementAsync.pending, function (state) {
            state.status = "loading";
        })
            .addCase(exports.incrementAsync.fulfilled, function (state, action) {
            state.status = "idle";
            state.value += action.payload;
        })
            .addCase(exports.incrementAsync.rejected, function (state) {
            state.status = "failed";
        });
    },
});
exports.increment = (_a = exports.counterSlice.actions, _a.increment), exports.decrement = _a.decrement, exports.incrementByAmount = _a.incrementByAmount;
// The function below is called a selector and allows us to select a value from
// the state. Selectors can also be defined inline where they"re used instead of
// in the slice file. For example: `useSelector((state: RootState) => state.counter.value)`
var selectCount = function (state) { return state.counter.value; };
exports.selectCount = selectCount;
// We can also write thunks by hand, which may contain both sync and async logic.
// Here"s an example of conditionally dispatching actions based on current state.
var incrementIfOdd = function (amount) {
    return function (dispatch, getState) {
        var currentValue = (0, exports.selectCount)(getState());
        if (currentValue % 2 === 1) {
            dispatch((0, exports.incrementByAmount)(amount));
        }
    };
};
exports.incrementIfOdd = incrementIfOdd;
exports.default = exports.counterSlice.reducer;
//# sourceMappingURL=counterSlice.js.map